@@ -5,6 +5,9 @@ from scipy.signal import medfilt2d, find_peaks
 from skimage.measure import label
 from skimage.morphology import closing, disk
 
+import multiprocessing as mp
+from itertools import repeat as irepeat
+
 from bmlab import Session
 from bmlab.fits import fit_vipa, VIPA, fit_lorentz_region
 from bmlab.image import extract_lines_along_arc, find_max_in_radius
@@ -70,13 +73,22 @@ class ExtractionController(object):
             return
 
         img = medfilt2d(img)
+        # This is the background level
         threshold = np.median(img)
-        img_closed = closing(img > (threshold + min_height), disk(10))
+
+        # Try to find a signal dependent estimate
+        # for the minimal peak height.
+        # Discard all values smaller than the background noise
+        # plus a minimal peak height
+        img_peaks = img[img > threshold + min_height]
+        # Calculate the signal dependent peak threshold
+        height = (np.nanmean(img_peaks) + threshold) / 2
+        img_closed = closing(img > height, disk(10))
 
         # Find all peaks higher than the min_height
-        all_peaks = []
         image_label, num = label(img_closed, return_num=True)
 
+        all_peaks = []
         for region in range(1, num + 1):
             # Mask of everything but the peak
             mask = (image_label != region)
@@ -270,6 +282,7 @@ class CalibrationController(object):
         cm.clear_brillouin_fits(calib_key)
         cm.clear_rayleigh_fits(calib_key)
         cm.clear_frequencies(calib_key)
+        cm.clear_vipa_params(calib_key)
 
         calculate_derived_values()
 
@@ -335,6 +348,17 @@ class CalibrationController(object):
             cm.set_spectra(calib_key, spectra)
         return spectra
 
+    def expected_frequencies(self, calib_key=None, current_frame=None):
+        cm = self.session.calibration_model()
+
+        if calib_key not in cm.vipa_params or \
+                current_frame > len(cm.vipa_params[calib_key]) - 1:
+            return None
+
+        return self.session.setup.calibration.shifts \
+            + self.session.setup.calibration.orders \
+            * cm.vipa_params[calib_key][current_frame][3]
+
 
 class PeakSelectionController(object):
 
@@ -424,6 +448,9 @@ class EvaluationController(object):
             'nr_rayleigh_regions': len(rayleigh_regions),
         })
 
+        pool_size = mp.cpu_count() * 2
+        pool = mp.Pool(processes=pool_size)
+
         # Loop over all measurement positions
         ind_total = 0
         for ind_x in range(resolution[0]):
@@ -445,34 +472,34 @@ class EvaluationController(object):
                     evm.results['intensity'][ind_x, ind_y, ind_z, :, 0, 0] =\
                         intensities
 
-                    for region_key, brillouin_region \
-                            in enumerate(brillouin_regions):
-
-                        brillouin_fits = \
-                            self.fit_spectra(spectra, brillouin_region)
-                        for frame_num, spectrum in enumerate(spectra):
-                            ind = (ind_x, ind_y, ind_z, frame_num,
-                                   region_key, 0)
+                    # Pack the data for parallel processing
+                    regions = brillouin_regions + rayleigh_regions
+                    packed_data = zip(irepeat(spectra), regions)
+                    # Process it
+                    results = pool.starmap(self.fit_spectra, packed_data)
+                    # Unpack the results
+                    for frame_num, spectrum in enumerate(spectra):
+                        for region_key, region in enumerate(brillouin_regions):
+                            ind = (ind_x, ind_y, ind_z,
+                                   frame_num, region_key, 0)
                             evm.results['brillouin_peak_position'][ind] =\
-                                brillouin_fits[frame_num][0]
+                                results[region_key][frame_num][0]
                             evm.results['brillouin_peak_fwhm'][ind] =\
-                                brillouin_fits[frame_num][1]
+                                results[region_key][frame_num][1]
                             evm.results['brillouin_peak_intensity'][ind] =\
-                                brillouin_fits[frame_num][2]
+                                results[region_key][frame_num][2]
 
-                    for region_key, rayleigh_region \
-                            in enumerate(rayleigh_regions):
-                        rayleigh_fits = \
-                            self.fit_spectra(spectra, rayleigh_region)
-                        for frame_num, spectrum in enumerate(spectra):
-
-                            ind = (ind_x, ind_y, ind_z, frame_num, region_key)
+                        for region_key, region \
+                                in enumerate(rayleigh_regions,
+                                             start=len(brillouin_regions)):
+                            ind = (ind_x, ind_y, ind_z, frame_num,
+                                   region_key - len(brillouin_regions))
                             evm.results['rayleigh_peak_position'][ind] =\
-                                rayleigh_fits[frame_num][0]
+                                results[region_key][frame_num][0]
                             evm.results['rayleigh_peak_fwhm'][ind] =\
-                                rayleigh_fits[frame_num][1]
+                                results[region_key][frame_num][1]
                             evm.results['rayleigh_peak_intensity'][ind] =\
-                                rayleigh_fits[frame_num][2]
+                                results[region_key][frame_num][2]
 
                     if count is not None:
                         count.value += 1
@@ -482,6 +509,8 @@ class EvaluationController(object):
                     if (ind_total % 10) == 0:
                         calculate_derived_values()
 
+        pool.close()
+        pool.join()
         calculate_derived_values()
 
         return
