@@ -114,6 +114,9 @@ class Session(Serializer):
         repetitions = self.file.repetition_keys()
         for repetition in repetitions:
             imgs = self.file.get_repetition(repetition).payload.get_image('0')
+            # If no images are available, skip this repetition
+            if imgs is None:
+                continue
             img = self.orientation.apply(imgs[0, ...])
             self.extraction_models.get(repetition).set_image_shape(img.shape)
 
@@ -181,10 +184,11 @@ class Session(Serializer):
         else:
             Session.get_instance().file = file
 
-    def get_calib_keys(self):
+    def get_calib_keys(self, sort_by_time=False):
         if self.current_repetition() is None:
             return None
-        return self.current_repetition().calibration.image_keys()
+        return self.current_repetition()\
+            .calibration.image_keys(sort_by_time=sort_by_time)
 
     def get_calibration_image(self, calib_key, frame_num=None):
         if self.current_repetition() is None:
@@ -194,6 +198,10 @@ class Session(Serializer):
             imgs = imgs[frame_num, ...]
         return self.orientation.apply(imgs)
 
+    def get_calibration_image_count(self, calib_key):
+        return self.current_repetition()\
+            .calibration.get_image_count(calib_key)
+
     def get_calibration_time(self, calib_key):
         if self.current_repetition() is None:
             return None
@@ -212,6 +220,9 @@ class Session(Serializer):
             imgs = imgs[frame_num, ...]
         return self.orientation.apply(imgs)
 
+    def get_payload_image_count(self, calib_key):
+        return self.current_repetition().payload.get_image_count(calib_key)
+
     def get_payload_time(self, image_key):
         if self.current_repetition() is None:
             return None
